import 'package:mobx/mobx.dart';
import 'package:snowtodoapp/data/helpers/extensions.dart';
import 'package:snowtodoapp/di/di.dart';
import 'package:snowtodoapp/domain/entities/todo_entity.dart';
import 'package:snowtodoapp/domain/usecases/auth/logout_user.dart';
import 'package:snowtodoapp/domain/usecases/todo_list/add_new_todo_item.dart';
import 'package:snowtodoapp/domain/usecases/todo_list/get_todo_list.dart';
import 'package:snowtodoapp/domain/usecases/todo_list/set_todo_item_as_finished.dart';

part 'home_controller.g.dart';

class HomeController = _HomeControllerBase with _$HomeController;

abstract class _HomeControllerBase with Store {
  _HomeControllerBase();

  final _getTodoListUseCase = getIt<GetTodoList>();
  final _setTodoItemAsFinishedUseCase = getIt<SetToDoItemAsFinished>();
  final _addNewToDoItemUseCase = getIt<AddNewToDoItem>();
  final logoutUser = getIt.get<LogoutUser>();

  @observable
  ObservableFuture<ObservableList<TodoItem>> todoList;

  var _todoItemTitle = "";

  @action
  Future<void> saveNewTodoItem() async {
    showingInputField = false;

    if (_todoItemTitle.isNotEmpty) {
      final newTodo = await _addNewToDoItemUseCase(TodoItem(
        title: _todoItemTitle,
        isDone: false,
      ));
      todoList.value.add(newTodo);
    }
  }

  @action
  Future<void> getTodoList() async {
    todoList = _getTodoListUseCase()
        .map((list) async => (await list).asObservable())
        .asObservable();
  }

  @action
  Future<void> setTodoItemAsFinished(TodoItem toDoItem) async {
    final newTodo = await _setTodoItemAsFinishedUseCase(toDoItem);
    final index = todoList.value.indexOf(toDoItem);
    todoList.value[index] = newTodo;
  }

  void onChanged(String value) {
    _todoItemTitle = value;
  }

  @action
  void newTodoItem() => showingInputField = true;

  @observable
  var showingInputField = false;

  void logout() => logoutUser();
}
