import 'package:injectable/injectable.dart';
import 'package:snowtodoapp/data/local/hive_client.dart';
import 'package:snowtodoapp/data/mappers/index.dart';
import 'package:snowtodoapp/domain/entities/todo_entity.dart';

@injectable
class TodoListLocalDataSource {
  final HiveClient _hiveClient;
  static const String box = "todo-items";

  TodoListLocalDataSource(this._hiveClient);

  Future<List<TodoItem>> getToDoList() async {
    final res = await _hiveClient.getAll(box);
    return res
        .map<TodoItem>((element) => TodoItem().fromJson(element))
        .toList();
  }

  Future<TodoItem> setTodoItemAsFinished(TodoItem todoItem) async {
    await _hiveClient.put(
      box,
      todoItem.id,
      todoItem.toJson,
    );
    return todoItem;
  }

  Future<TodoItem> addNewToDoItem(TodoItem toDoItem) async {
    final res = await _hiveClient.add(box, toDoItem.toJson);
    return toDoItem.copyWith(id: res.toString());
  }
}
