import 'package:injectable/injectable.dart';
import 'package:snowtodoapp/data/mappers/index.dart';
import 'package:snowtodoapp/data/remote/dio_client.dart';
import 'package:snowtodoapp/domain/entities/todo_entity.dart';

@injectable
class TodoListRemoteDataSource {
  final DioClient _dioClient;

  TodoListRemoteDataSource(this._dioClient);

  Future<List<TodoItem>> getToDoList() async {
    final response = await _dioClient.get<List>('/todo-items');
    return response
        .map<TodoItem>((element) => TodoItem().fromJson(element))
        .toList();
  }

  Future<TodoItem> setTodoItemAsFinished(TodoItem todoItem) async {
    final res = await _dioClient.put(
      "/todo-items/${todoItem.id}",
      data: todoItem.toJson,
    );
    return TodoItem().fromJson(res);
  }

  Future<TodoItem> addNewToDoItem(TodoItem toDoItem) async {
    final res = await _dioClient.post("/todo-items", data: toDoItem.toJson);
    return TodoItem().fromJson(res);
  }
}
