import 'dart:async';

import 'package:dio/dio.dart';
import 'package:projectbasesnow/projectbasesnow.dart';

class ErrorMapper {
  static FutureOr<T> request<T>(FutureOr<T> Function() req) async {
    try {
      final res = await req();
      return res;
    } on DioError catch (e) {
      return Future.error(
        AppException(
          exception: e,
          message: _dioError(e),
        ),
      );
    } on AppException catch (e) {
      return Future.error(e);
    } catch (e) {
      return Future.error(
        AppException(
          exception: e,
          message: "Erro desconhecido",
        ),
      );
    }
  }

  static String _dioError(DioError error) {
    switch (error.type) {
      case DioErrorType.SEND_TIMEOUT:
      case DioErrorType.CONNECT_TIMEOUT:
      case DioErrorType.RECEIVE_TIMEOUT:
        return "Falha de conexão, verifique sua internet";
        break;
      case DioErrorType.CANCEL:
        return "Requisição cancelada";
        break;
      case DioErrorType.RESPONSE:
      case DioErrorType.DEFAULT:
      default:
        break;
    }
    if (error.response?.statusCode != null) {
      switch (error.response.statusCode) {
        case 401:
          return "Autorização negada, verifique seu login";
          break;
        case 403:
          return "Ocorreu um erro na sua requisição, verifique os dados e tente novamente";
          break;
        case 404:
          return "Não encontrado";
          break;
        case 500:
          return "Erro interno do servidor";
          break;
        case 503:
          return "O servidor está indisponível no momento, tente novamente";
          break;
        default:
      }
    }
    return "Erro na requisição, tente novamente";
  }
}
