import 'package:injectable/injectable.dart';
import 'package:snowtodoapp/data/data_sources/todo_list/todo_list_local_data_source.dart';
import 'package:snowtodoapp/data/data_sources/todo_list/todo_list_remote_data_source.dart';
import 'package:snowtodoapp/domain/entities/todo_entity.dart';
import 'package:snowtodoapp/domain/repositories/todo_list/todo_list_repository.dart';

@Injectable(as: TodoListRepository)
class TodoListRepositoryImpl implements TodoListRepository {
  final TodoListRemoteDataSource _remoteDataSource;
  final TodoListLocalDataSource _localDataSource;

  TodoListRepositoryImpl(this._remoteDataSource, this._localDataSource);

  @override
  Future<List<TodoItem>> getToDoList() async {
    return _localDataSource.getToDoList();
  }

  @override
  Future<TodoItem> setTodoItemAsFinished(TodoItem toDoItem) async {
    return _localDataSource
        .setTodoItemAsFinished(toDoItem.copyWith(isDone: !toDoItem.isDone));
  }

  @override
  Future<TodoItem> addNewToDoItem(TodoItem toDoItem) async {
    return _localDataSource.addNewToDoItem(toDoItem);
  }
}
